package com.paxos.common

import java.util.UUID
import kotlin.time.Duration

abstract class Wrapper<T, W : Wrapper<T, W>>(val value: T) {
    override fun toString() = value.toString()
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is Wrapper<*, *>) return false

        if (value != other.value) return false

        return true
    }

    override fun hashCode() = value?.hashCode() ?: 0
}

abstract class ComparableWrapper<T : Comparable<T>, W : ComparableWrapper<T, W>>(id: T) : Wrapper<T, W>(id), Comparable<W> {
    override fun compareTo(other: W) = value.compareTo(other.value)
}

abstract class StringWrapper<T : StringWrapper<T>>(id: String) : ComparableWrapper<String, T>(id)
abstract class IntWrapper<T : IntWrapper<T>>(id: Int) : ComparableWrapper<Int, T>(id)
abstract class DurationWrapper<T : DurationWrapper<T>>(id: Duration) : ComparableWrapper<Duration, T>(id)
abstract class BooleanWrapper<T : BooleanWrapper<T>>(id: Boolean) : ComparableWrapper<Boolean, T>(id)
abstract class UuidWrapper<T : UuidWrapper<T>>(id: UUID) : ComparableWrapper<UUID, T>(id)
